@@ -92,4 +92,14 @@ static inline int usb_translate_errors(int error_code)
 }
 #endif /* LINUX_VERSION_IS_LESS(3,2,0) */
 
+#if LINUX_VERSION_IS_LESS(4,12,0)
+#define usb_find_common_endpoints LINUX_BACKPORT(usb_find_common_endpoints)
+int __must_check
+usb_find_common_endpoints(struct usb_host_interface *alt,
+		struct usb_endpoint_descriptor **bulk_in,
+		struct usb_endpoint_descriptor **bulk_out,
+		struct usb_endpoint_descriptor **int_in,
+		struct usb_endpoint_descriptor **int_out);
+#endif /* < 4.12 */
+
 #endif /* __BACKPORT_USB_H */
