@@ -761,7 +761,7 @@ static void hostap_set_multicast_list(struct net_device *dev)
 }
 
 
-static void prism2_tx_timeout(struct net_device *dev)
+static void prism2_tx_timeout(struct net_device *dev, unsigned int txqueue)
 {
 	struct hostap_interface *iface;
 	local_info_t *local;
@@ -781,6 +781,15 @@ static void prism2_tx_timeout(struct net_device *dev)
 
 	local->func->schedule_reset(local);
 }
+#if LINUX_VERSION_IS_LESS(5,6,0)
+/* Just declare it here to keep sparse happy */
+void bp_prism2_tx_timeout(struct net_device *dev);
+void bp_prism2_tx_timeout(struct net_device *dev)
+{
+	prism2_tx_timeout(dev, 0);
+}
+EXPORT_SYMBOL_GPL(bp_prism2_tx_timeout);
+#endif
 
 const struct header_ops hostap_80211_ops = {
 	.create		= eth_header,
@@ -804,7 +813,7 @@ static const struct net_device_ops hostap_netdev_ops = {
 #if LINUX_VERSION_IS_LESS(4,10,0)
 	.ndo_change_mtu = __change_mtu,
 #endif
-
+	
 	.ndo_start_xmit		= hostap_data_start_xmit,
 
 	.ndo_open		= prism2_open,
@@ -812,7 +821,12 @@ static const struct net_device_ops hostap_netdev_ops = {
 	.ndo_do_ioctl		= hostap_ioctl,
 	.ndo_set_mac_address	= prism2_set_mac_address,
 	.ndo_set_rx_mode	= hostap_set_multicast_list,
+#if LINUX_VERSION_IS_GEQ(5,6,0)
 	.ndo_tx_timeout 	= prism2_tx_timeout,
+#else
+	.ndo_tx_timeout = bp_prism2_tx_timeout,
+#endif
+	
 	.ndo_validate_addr	= eth_validate_addr,
 };
 
@@ -820,7 +834,7 @@ static const struct net_device_ops hostap_mgmt_netdev_ops = {
 #if LINUX_VERSION_IS_LESS(4,10,0)
 	.ndo_change_mtu = __change_mtu,
 #endif
-
+	
 	.ndo_start_xmit		= hostap_mgmt_start_xmit,
 
 	.ndo_open		= prism2_open,
@@ -828,7 +842,12 @@ static const struct net_device_ops hostap_mgmt_netdev_ops = {
 	.ndo_do_ioctl		= hostap_ioctl,
 	.ndo_set_mac_address	= prism2_set_mac_address,
 	.ndo_set_rx_mode	= hostap_set_multicast_list,
+#if LINUX_VERSION_IS_GEQ(5,6,0)
 	.ndo_tx_timeout 	= prism2_tx_timeout,
+#else
+	.ndo_tx_timeout = bp_prism2_tx_timeout,
+#endif
+	
 	.ndo_validate_addr	= eth_validate_addr,
 };
 
@@ -836,7 +855,7 @@ static const struct net_device_ops hostap_master_ops = {
 #if LINUX_VERSION_IS_LESS(4,10,0)
 	.ndo_change_mtu = __change_mtu,
 #endif
-
+	
 	.ndo_start_xmit 	= hostap_master_start_xmit,
 
 	.ndo_open		= prism2_open,
@@ -844,7 +863,12 @@ static const struct net_device_ops hostap_master_ops = {
 	.ndo_do_ioctl		= hostap_ioctl,
 	.ndo_set_mac_address	= prism2_set_mac_address,
 	.ndo_set_rx_mode	= hostap_set_multicast_list,
+#if LINUX_VERSION_IS_GEQ(5,6,0)
 	.ndo_tx_timeout 	= prism2_tx_timeout,
+#else
+	.ndo_tx_timeout = bp_prism2_tx_timeout,
+#endif
+	
 	.ndo_validate_addr	= eth_validate_addr,
 };
 
@@ -880,7 +904,7 @@ void hostap_setup_dev(struct net_device *dev, local_info_t *local,
 #else
 		dev->tx_queue_len = 0;
 #endif
-	/* use main radio device queue */
+			/* use main radio device queue */
 		dev->netdev_ops = &hostap_mgmt_netdev_ops;
 		dev->type = ARPHRD_IEEE80211;
 		dev->header_ops = &hostap_80211_ops;
@@ -894,7 +918,7 @@ void hostap_setup_dev(struct net_device *dev, local_info_t *local,
 #else
 		dev->tx_queue_len = 0;
 #endif
-	/* use main radio device queue */
+			/* use main radio device queue */
 		dev->netdev_ops = &hostap_netdev_ops;
 	}
 
